#
# Copyright (c) 2019-2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
# LicenseRef-NvidiaProprietary
#
# NVIDIA CORPORATION, its affiliates and licensors retain all intellectual
# property and proprietary rights in and to this material, related
# documentation and any modifications thereto. Any use, reproduction,
# disclosure or distribution of this material and related documentation
# without an express license agreement from NVIDIA CORPORATION or
# its affiliates is strictly prohibited
#


import os

from netq_lib.orm import HOSTNAME
from netq_lib.orm.redisdb.models import ClSupport
from netq_lib.common.enums import ServiceStatus
from netq_lib.common.utils import save_fullstate
from netq_agent.parsers.common.render import BaseRender


class ClSupportRender(BaseRender):
    def __init__(self):
        super(ClSupportRender, self).__init__()

    def build_fullstate_clobj(self, file_name, current_cl_support_objs):
        new_cl_support_objs = ClSupport()
        new_cl_support_objs.file_abs_name = file_name
        new_cl_support_objs.create_time = os.path.getctime(file_name)
        current_cl_support_objs.append(new_cl_support_objs)

    def parse(self, cmd_output, svc_failed=False, **kwargs):
        filepath = self._get_filepath()
        file_to_monitor_start_name = self._get_file_start_name()
        file_to_monitor_end_name = self._get_file_end_name()

        fullstate = kwargs.get('fullstate')

        # grab current list of clSupport objects
        m_clsupport = ClSupport.memmodel()
        old_cl_support_objs = []
        current_cl_support_objs = []
        if m_clsupport:
            old_cl_support_objs = [old_obj for old_obj in m_clsupport.query.filter(hostname=HOSTNAME)]

        if os.path.isdir(filepath):
            for current_entry in [name for name in os.listdir(filepath) if not os.path.isdir(os.path.join(filepath, name))]:
                new_file_name = filepath + "/" + current_entry
                if fullstate:
                    self.build_fullstate_clobj(new_file_name, current_cl_support_objs)

                elif current_entry.startswith(file_to_monitor_start_name) and current_entry.endswith(file_to_monitor_end_name):
                    cl_support_obj = ClSupport()
                    cl_support_obj.file_abs_name = new_file_name
                    cl_support_obj.create_time = os.path.getctime(new_file_name)
                    current_cl_support_objs.append(cl_support_obj)

        for old_file_obj in old_cl_support_objs:
            if old_file_obj.file_abs_name not in [cl_obj.file_abs_name for cl_obj in current_cl_support_objs]:
                old_file_ent = ClSupport()
                old_file_ent.copy(old_file_obj)
                old_file_ent.delete()
                old_cl_support_objs.remove(old_file_obj)


        old_cl_support_filenames = [cl_obj.file_abs_name for cl_obj in old_cl_support_objs]
        for cur_file_obj in current_cl_support_objs:
            if cur_file_obj.file_abs_name not in old_cl_support_filenames:
                cur_file_obj.save()

        if fullstate:
            save_fullstate(current_cl_support_objs, ClSupport().__class__.__name__)

        return ServiceStatus.OK